==> 0.0.10.tar.gz: lib/widgets/fields/text_field.dart <==
import 'package:extensions_pack/extensions_pack.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:rxdart/rxdart.dart';
import 'package:widgets_pack/widgets_pack.dart';

enum AppTextFormFieldLabelBehavior {
  above,
  flutterAuto,
  flutterAlways;

  bool get isFlutter => name.contains('flutter');

  FloatingLabelBehavior? flutterBehavior() {
    return switch (this) {
      (AppTextFormFieldLabelBehavior.flutterAuto) => FloatingLabelBehavior.auto,
      (AppTextFormFieldLabelBehavior.flutterAlways) => FloatingLabelBehavior.always,
      (_) => null,
    };
  }
}

class AppTextFormField extends StatefulWidget {
  static String tapRegionGroupId = 'AppTextFieldGroupId';

  final List<String>? autofillHints;
  final AutovalidateMode autoValidateMode;
  final InputBorder? border;
  final EdgeInsets? contentPadding;
  final TextEditingController? controller;
  final Duration debounceTime;
  final bool enabled;
  final InputBorder? enabledBorder;
  final Color? fillColor;
  final bool? filled;
  final InputBorder? focusedBorder;
  final FocusNode? focusNode;
  final String? helperText;
  final String? hintText;
  final TextStyle? hintStyle;
  final String? initialValue;
  final List<TextInputFormatter>? inputFormatters;
  final InputDecorationTheme? inputTheme;
  final TextInputType? keyboardType;
  final AppTextFormFieldLabelBehavior labelBehavior;
  final String? labelText;
  final int? maxLength;

  /// Value is ignored if [obscureText] is true
  final int? maxLines;

  final int? minLines;
  final bool obscureText;
  final ValueChanged<String>? onChanged;
  final ValueChanged<String>? onFieldSubmitted;
  final Widget? prefixIcon;
  final String? prefixText;
  final bool readOnly;
  final bool requestFocusOnInitState;
  final bool showErrorString;
  final TextStyle? style;

  /// Value is ignored if [obscureText] is true
  final Widget? suffixIcon;

  final Widget? suffix;
  final String? suffixText;
  final TextStyle? suffixStyle;
  final TextInputAction? textInputAction;
  final FormFieldValidator<String>? validator;

  const AppTextFormField({
    this.autofillHints,
    this.autoValidateMode = AutovalidateMode.onUserInteraction,
    this.border,
    this.contentPadding,
    this.controller,
    this.debounceTime = Duration.zero,
    this.enabled = true,
    this.enabledBorder,
    this.fillColor,
    this.filled,
    this.focusedBorder,
    this.focusNode,
    this.helperText,
    this.hintText,
    this.hintStyle,
    this.initialValue,
    this.inputFormatters,
    this.inputTheme,
    this.keyboardType,
    this.labelBehavior = AppTextFormFieldLabelBehavior.flutterAuto,
    this.labelText,
    this.maxLength,
    this.maxLines = 1,
    this.minLines,
    this.obscureText = false,
    this.onChanged,
    this.onFieldSubmitted,
    this.prefixIcon,
    this.prefixText,
    this.readOnly = false,
    this.requestFocusOnInitState = false,
    this.showErrorString = true,
    this.style,
    this.suffix,
    this.suffixIcon,
    this.suffixText,
    this.suffixStyle,
    this.textInputAction,
    this.validator,
    super.key,
  });

  AppTextFormField.search({
    this.border,
    this.contentPadding,
    this.controller,
    this.fillColor,
    this.filled,
    this.focusedBorder,
    this.focusNode,
    this.helperText,
    this.hintText = 'Search',
    this.initialValue,
    this.inputTheme,
    this.onChanged,
    this.onFieldSubmitted,
    this.requestFocusOnInitState = false,
    super.key,
  })  : autofillHints = null,
        autoValidateMode = AutovalidateMode.disabled,
        debounceTime = defaultDebounceTime,
        enabled = true,
        enabledBorder = null,
        hintStyle = null,
        inputFormatters = null,
        keyboardType = TextInputType.text,
        labelText = null,
        labelBehavior = AppTextFormFieldLabelBehavior.above,
        maxLength = null,
        maxLines = 1,
        minLines = null,
        obscureText = false,
        prefixIcon = const Icon(
          Icons.search,
          size: kMDSize,
        ),
        prefixText = null,
        readOnly = false,
        showErrorString = false,
        style = null,
        suffix = null,
        suffixIcon = null,
        suffixText = null,
        suffixStyle = null,
        textInputAction = TextInputAction.search,
        validator = null;

  static Duration get defaultDebounceTime => const Duration(milliseconds: 350);

  @override
  State<AppTextFormField> createState() => _AppTextFormFieldState();
}

class _AppTextFormFieldState extends State<AppTextFormField> {
  final _debounceSearch = BehaviorSubject<String>();
  final _inputKey = GlobalKey<FormFieldState<String>>();

  late final _obscureTextNotifier = ValueNotifier<bool>(widget.obscureText);
  late final _focusNode = widget.focusNode ?? FocusNode();
  late final _controller = widget.controller ?? TextEditingController();

  @override
  void initState() {
    super.initState();

    if (widget.initialValue != null) {
      _controller.text = widget.initialValue!;
    }

    _debounceSearch
        .distinct()
        .debounceTime(widget.debounceTime)
        .listen((event) => widget.onChanged?.call(_controller.text));

    if (widget.requestFocusOnInitState) {
      _focusNode.requestFocus();
    }
  }

  @override
  void didUpdateWidget(covariant AppTextFormField oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (widget.initialValue != null && widget.initialValue != oldWidget.initialValue && !_focusNode.hasFocus) {
      _controller.text = widget.initialValue!;
    }
  }

  @override
  void dispose() {
    super.dispose();

    _debounceSearch.close();
    _obscureTextNotifier.dispose();

    if (widget.controller == null) _controller.dispose();
    if (widget.focusNode == null) _focusNode.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final inputTheme = widget.inputTheme ?? Theme.of(context).inputDecorationTheme;

    return TapRegion(
      groupId: AppTextFormField.tapRegionGroupId,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: [
          if (widget.labelText != null && !widget.labelBehavior.isFlutter) ...[
            TitleMedium(widget.labelText!),
            const Spacing(mainAxisExtent: kXXSSize),
          ],
          ValueListenableBuilder<bool>(
            valueListenable: _obscureTextNotifier,
            builder: (context, obscuring, _) {
              return TextFormField(
                key: _inputKey,
                autofillHints: widget.autofillHints,
                autovalidateMode: widget.autoValidateMode,
                controller: _controller,
                enabled: widget.enabled,
                focusNode: _focusNode,
                readOnly: widget.readOnly,
                maxLines: widget.obscureText ? 1 : widget.maxLines,
                minLines: widget.minLines,
                onChanged: _debounceSearch.add,
                obscureText: obscuring,
                onTapOutside: (_) => _focusNode.unfocus(),
                validator: widget.validator?.call,
                style: (widget.style ?? Theme.of(context).textTheme.bodyLarge),
                keyboardType: widget.keyboardType,
                inputFormatters: widget.inputFormatters,
                onFieldSubmitted: widget.onFieldSubmitted,
                textInputAction: widget.textInputAction,
                maxLength: widget.maxLength,
                decoration: InputDecoration(
                  contentPadding: widget.contentPadding,
                  errorStyle: widget.showErrorString ? null : const TextStyle(fontSize: 0),
                  enabledBorder: widget.enabledBorder ?? widget.border,
                  suffixIcon: widget.obscureText
                      ? GestureDetector(
                          onTap: () {
                            _obscureTextNotifier.value = !obscuring;
                          },
                          child: Icon(
                            obscuring ? Icons.visibility : Icons.visibility_off,
                          ),
                        )
                      : widget.suffixIcon,
                  suffix: widget.suffix,
                  helperText: widget.helperText,
                  hintText: widget.hintText,
                  hintStyle: widget.hintStyle,
                  prefixIcon: widget.prefixIcon,
                  prefixText: widget.prefixText,
                  suffixText: widget.suffixText,
                  suffixStyle: widget.suffixStyle,
                  focusedBorder: widget.focusedBorder ?? widget.border,
                  fillColor: widget.fillColor,
                  filled: widget.filled,
                  labelText: widget.labelBehavior.isFlutter ? widget.labelText : null,
                  floatingLabelBehavior: widget.labelBehavior.flutterBehavior(),
                ).applyDefaults(inputTheme),
              );
            },
          ),
        ],
      ),
    );
  }
}

==> 0.0.19.tar.gz: lib/widgets/fields/text_field.dart <==
import 'package:extensions_pack/extensions_pack.dart';
import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:rxdart/rxdart.dart';
import 'package:widgets_pack/widgets_pack.dart';

enum AppTextFormFieldLabelBehavior {
  above,
  flutterAuto,
  flutterAlways;

  bool get isFlutter => name.contains('flutter');

  FloatingLabelBehavior? flutterBehavior() {
    return switch (this) {
      (AppTextFormFieldLabelBehavior.flutterAuto) => FloatingLabelBehavior.auto,
      (AppTextFormFieldLabelBehavior.flutterAlways) => FloatingLabelBehavior.always,
      (_) => null,
    };
  }
}

class AppTextFormField extends StatefulWidget {
  static String tapRegionGroupId = 'AppTextFieldGroupId';

  final List<String>? autofillHints;
  final AutovalidateMode autoValidateMode;
  final InputBorder? border;
  final EdgeInsets? contentPadding;
  final TextEditingController? controller;
  final Duration debounceTime;
  final bool enabled;
  final InputBorder? enabledBorder;
  final Color? fillColor;
  final bool? filled;
  final InputBorder? focusedBorder;
  final FocusNode? focusNode;
  final String? helperText;
  final String? hintText;
  final TextStyle? hintStyle;
  final String? initialValue;
  final List<TextInputFormatter>? inputFormatters;
  final InputDecorationTheme? inputTheme;
  final TextInputType? keyboardType;
  final AppTextFormFieldLabelBehavior labelBehavior;
  final String? labelText;
  final int? maxLength;

  /// Value is ignored if [obscureText] is true
  final int? maxLines;

  final int? minLines;
  final bool obscureText;
  final ValueChanged<String>? onChanged;
  final ValueChanged<String>? onFieldSubmitted;
  final Widget? prefixIcon;
  final String? prefixText;
  final bool readOnly;
  final bool requestFocusOnInitState;
  final bool showErrorString;
  final TextStyle? style;

  /// Value is ignored if [obscureText] is true
  final Widget? suffixIcon;

  final Widget? suffix;
  final String? suffixText;
  final TextStyle? suffixStyle;
  final TextInputAction? textInputAction;
  final FormFieldValidator<String>? validator;

  const AppTextFormField({
    this.autofillHints,
    this.autoValidateMode = AutovalidateMode.disabled,
    this.border,
    this.contentPadding,
    this.controller,
    this.debounceTime = Duration.zero,
    this.enabled = true,
    this.enabledBorder,
    this.fillColor,
    this.filled,
    this.focusedBorder,
    this.focusNode,
    this.helperText,
    this.hintText,
    this.hintStyle,
    this.initialValue,
    this.inputFormatters,
    this.inputTheme,
    this.keyboardType,
    this.labelBehavior = AppTextFormFieldLabelBehavior.flutterAuto,
    this.labelText,
    this.maxLength,
    this.maxLines = 1,
    this.minLines,
    this.obscureText = false,
    this.onChanged,
    this.onFieldSubmitted,
    this.prefixIcon,
    this.prefixText,
    this.readOnly = false,
    this.requestFocusOnInitState = false,
    this.showErrorString = true,
    this.style,
    this.suffix,
    this.suffixIcon,
    this.suffixText,
    this.suffixStyle,
    this.textInputAction,
    this.validator,
    super.key,
  });

  AppTextFormField.search({
    this.border,
    this.contentPadding,
    this.controller,
    this.fillColor,
    this.filled,
    this.focusedBorder,
    this.focusNode,
    this.helperText,
    this.hintText = 'Search',
    this.initialValue,
    this.inputTheme,
    this.onChanged,
    this.onFieldSubmitted,
    this.requestFocusOnInitState = false,
    super.key,
  })  : autofillHints = null,
        autoValidateMode = AutovalidateMode.disabled,
        debounceTime = defaultDebounceTime,
        enabled = true,
        enabledBorder = null,
        hintStyle = null,
        inputFormatters = null,
        keyboardType = TextInputType.text,
        labelText = null,
        labelBehavior = AppTextFormFieldLabelBehavior.above,
        maxLength = null,
        maxLines = 1,
        minLines = null,
        obscureText = false,
        prefixIcon = const Icon(
          Icons.search,
          size: kMDSize,
        ),
        prefixText = null,
        readOnly = false,
        showErrorString = false,
        style = null,
        suffix = null,
        suffixIcon = null,
        suffixText = null,
        suffixStyle = null,
        textInputAction = TextInputAction.search,
        validator = null;

  static Duration get defaultDebounceTime => const Duration(milliseconds: 350);

  @override
  State<AppTextFormField> createState() => _AppTextFormFieldState();
}

class _AppTextFormFieldState extends State<AppTextFormField> {
  final _debounceSearch = BehaviorSubject<String>();
  final _inputKey = GlobalKey<FormFieldState<String>>();

  late final _obscureTextNotifier = ValueNotifier<bool>(widget.obscureText);
  late final _focusNode = widget.focusNode ?? FocusNode();
  late final _controller = widget.controller ?? TextEditingController();

  @override
  void initState() {
    super.initState();

    if (widget.initialValue != null) {
      _controller.text = widget.initialValue!;
    }

    _debounceSearch
        .distinct()
        .debounceTime(widget.debounceTime)
        .listen((event) => widget.onChanged?.call(_controller.text));

    if (widget.requestFocusOnInitState) {
      _focusNode.requestFocus();
    }
  }

  @override
  void didUpdateWidget(covariant AppTextFormField oldWidget) {
    super.didUpdateWidget(oldWidget);

    if (widget.initialValue != null && widget.initialValue != oldWidget.initialValue && !_focusNode.hasFocus) {
      _controller.text = widget.initialValue!;
    }
  }

  @override
  void dispose() {
    super.dispose();

    _debounceSearch.close();
    _obscureTextNotifier.dispose();

    if (widget.controller == null) _controller.dispose();
    if (widget.focusNode == null) _focusNode.dispose();
  }

  @override
  Widget build(BuildContext context) {
    final inputTheme = widget.inputTheme ?? Theme.of(context).inputDecorationTheme;

    return TapRegion(
      groupId: AppTextFormField.tapRegionGroupId,
      child: Column(
        crossAxisAlignment: CrossAxisAlignment.start,
        mainAxisSize: MainAxisSize.min,
        children: [
          if (widget.labelText != null && !widget.labelBehavior.isFlutter) ...[
            TitleMedium(widget.labelText!),
            const Spacing(mainAxisExtent: kXXSSize),
          ],
          ValueListenableBuilder<bool>(
            valueListenable: _obscureTextNotifier,
            builder: (context, obscuring, _) {
              return TextFormField(
                key: _inputKey,
                autofillHints: widget.autofillHints,
                autovalidateMode: widget.autoValidateMode,
                controller: _controller,
                enabled: widget.enabled,
                focusNode: _focusNode,
                readOnly: widget.readOnly,
                maxLines: widget.obscureText ? 1 : widget.maxLines,
                minLines: widget.minLines,
                onChanged: _debounceSearch.add,
                obscureText: obscuring,
                onTapOutside: (_) => _focusNode.unfocus(),
                validator: widget.validator?.call,
                style: (widget.style ?? Theme.of(context).textTheme.bodyLarge),
                keyboardType: widget.keyboardType,
                inputFormatters: widget.inputFormatters,
                onFieldSubmitted: widget.onFieldSubmitted,
                textInputAction: widget.textInputAction,
                maxLength: widget.maxLength,
                decoration: InputDecoration(
                  contentPadding: widget.contentPadding,
                  errorStyle: widget.showErrorString ? null : const TextStyle(fontSize: 0),
                  enabledBorder: widget.enabledBorder ?? widget.border,
                  suffixIcon: widget.obscureText
                      ? GestureDetector(
                          onTap: () {
                            _obscureTextNotifier.value = !obscuring;
                          },
                          child: Icon(
                            obscuring ? Icons.visibility : Icons.visibility_off,
                          ),
                        )
                      : widget.suffixIcon,
                  suffix: widget.suffix,
                  helperText: widget.helperText,
                  hintText: widget.hintText,
                  hintStyle: widget.hintStyle,
                  prefixIcon: widget.prefixIcon,
                  prefixText: widget.prefixText,
                  suffixText: widget.suffixText,
                  suffixStyle: widget.suffixStyle,
                  focusedBorder: widget.focusedBorder ?? widget.border,
                  fillColor: widget.fillColor,
                  filled: widget.filled,
                  labelText: widget.labelBehavior.isFlutter ? widget.labelText : null,
                  floatingLabelBehavior: widget.labelBehavior.flutterBehavior(),
                ).applyDefaults(inputTheme),
              );
            },
          ),
        ],
      ),
    );
  }
}
